@@ -6,22 +6,18 @@ GNU General Public License v3.0 only
 SPDX-License-Identifier: GPL-3.0-only
 """
 
-import asyncio
 import json
-import logging
-import sys
 import time
-
-from typing import Optional, NamedTuple
+from typing import Optional
 
 import aiosqlite
 
+import pyDE1
 from pyDE1.de1.profile import Profile
-from pyDE1.event_manager import EventNotificationName, SequencerGateName
-from pyDE1.event_manager.event_manager import EventNotificationAction
+from pyDE1.event_manager import SequencerGateName
+from pyDE1.event_manager.payloads import EventNotificationAction
 
-logger = logging.getLogger('DB.insert')
-logging.getLogger('aiosqlite').setLevel(logging.INFO)
+logger = pyDE1.getLogger('Database.Insert')
 
 
 async def profile(profile: Profile, db: aiosqlite.Connection, when: float):
@@ -42,6 +38,14 @@ async def profile(profile: Profile, db: aiosqlite.Connection, when: float):
         vals['date_added'] = when
         vals['source_format'] = vals['source_format'].value
         cur: aiosqlite.Cursor = await db.execute(sql, vals)
+        await db.commit()
+        logger.info(f"Profile {profile.id} added to profile table.")
+
+
+async def persist_last_profile(profile: Profile, db: aiosqlite.Connection,
+                               when: Optional[float] = None):
+    if not when:
+        when = time.time()
     # Update the last_profile information for persistence
     sql = "UPDATE persist_hkv " \
           "SET value=? " \
@@ -50,7 +54,7 @@ async def profile(profile: Profile, db: aiosqlite.Connection, when: float):
     sql = "UPDATE persist_hkv " \
           "SET value=? " \
           "WHERE header='last_profile' AND key='datetime'"
-    await db.execute(sql, (time.time(),))
+    await db.execute(sql, (when,))
     await db.commit()
 
 
