@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021, 2022 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -703,6 +703,8 @@ class API_Substates (enum.IntEnum):
     CleanGroup          = 0x10
     PausedRefill        = 0x11
     PausedSteam         = 0x12
+    UserNotPresent      = 0x13  # FW 1315/1316
+    SteamPuff           = 0x14  # FW 1315/1316
 
     Error_NaN           = 200
     Error_Inf           = 201
@@ -2593,6 +2595,8 @@ class MMRV13ModelCode (enum.IntEnum):
     DE1PRO      =  3
     DE1XL       =  4
     DE1CAFE     =  5
+    DE1XXL      =  6
+    UNRECOGNIZED = -1
 
 
 class MMRGHCInfoBitMask (enum.IntFlag):
@@ -2752,7 +2756,13 @@ def decode_one_mmr(addr_high: int, addr_low: Union[MMR0x80LowAddr, int],
         retval = mmr_bytes.decode('utf-8')
 
     elif addr_low == MMR0x80LowAddr.V13_MODEL:
-        retval = MMRV13ModelCode(unpack('<I', mmr_bytes)[0])
+        # New model codes shouldn't cause failure
+        try:
+            retval = MMRV13ModelCode(unpack('<I', mmr_bytes)[0])
+        except ValueError as e:
+            logger.warning(f"Unrecognized model: {e}")
+            retval = MMRV13ModelCode.UNRECOGNIZED
+
 
     elif addr_low == MMR0x80LowAddr.GHC_INFO:
         val = unpack('<I', mmr_bytes)[0]
@@ -2764,8 +2774,17 @@ def decode_one_mmr(addr_high: int, addr_low: Union[MMR0x80LowAddr, int],
     elif addr_low in {
         MMR0x80LowAddr.PREF_GHC_MCI,
         MMR0x80LowAddr.MAX_SHOT_PRESS,
-    } or addr_low > MMR0x80LowAddr.LAST_KNOWN:
-        # These aren't implemented (fully)
+    }:
+        # These aren't fully implemented (returning 0x0 at FW 1283)
+        retval = unpack('<I', mmr_bytes)[0]
+        logger.info(
+            "Unexpected decode requested for "
+            f"{addr_low.name} {retval} 0x{retval:x} "
+            "(unimplemented MMR)"
+        )
+
+    elif addr_low > MMR0x80LowAddr.LAST_KNOWN:
+        # These shouldn't be being read
         retval = unpack('<I', mmr_bytes)[0]
         logger.warning(
             "Unexpected decode requested for "
