@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021-2022 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -332,7 +332,11 @@ class DE1 (Singleton):
     # Self-contained calls for API
     #
 
-    async def first_if_found(self, doit: bool):
+    async def first_if_found(self, doit: bool, warn_deprecated=True):
+        if warn_deprecated:
+            logger.warning(
+                "Use of 'first_if_found' is deprecated in favor of "
+                "setting {'id': 'scan'}")
         if self.is_connected:
             logger.warning(
                 "first_if_found requested, but already connected. "
@@ -340,6 +344,7 @@ class DE1 (Singleton):
         elif not doit:
             logger.warning(
                 "first_if_found requested, but not True. No action taken.")
+
         else:
             device = await find_first_matching(('DE1',))
             if device:
@@ -356,6 +361,10 @@ class DE1 (Singleton):
 
         de1 = DE1()
 
+        if ble_device_id == 'scan':
+            await self.first_if_found(doit=True, warn_deprecated=False)
+            return
+
         # TODO: Need to make distasteful assumption that the id is the address
         try:
             if self.address == ble_device_id:
@@ -419,7 +428,8 @@ class DE1 (Singleton):
         if timeout is None:
             timeout = config.bluetooth.CONNECT_TIMEOUT
 
-        logger.info(f"Connecting to DE1 at {self.address}")
+        if self._log_reconnect_attempts:
+            logger.info(f"Connecting to DE1 at {self.address}")
 
         assert self._bleak_client is not None
 
@@ -449,7 +459,8 @@ class DE1 (Singleton):
                 asyncio.create_task(self.initialize_after_connection())
 
             else:
-                logger.error(f"Connection failed to DE1 at {self.address}")
+                if self._log_reconnect_attempts:
+                    logger.error(f"Connection failed to DE1 at {self.address}")
                 await self._notify_not_ready()
                 await self._event_connectivity.publish(
                     self._connectivity_change(
@@ -762,7 +773,7 @@ class DE1 (Singleton):
                 timeout=config.de1.CUUID_LOCK_WAIT_TIMEOUT)
         except asyncio.TimeoutError as e:
             cuuid_logger.critical(
-                "Timeout waiting for lock. Aborting process.")
+                "Timeout waiting for lock or request/notify. Aborting process.")
             raise e
 
         return retval
@@ -892,7 +903,6 @@ class DE1 (Singleton):
         return event_list, addr_low_list
 
 
-
     #
     # Upload a shot profile
     #
@@ -901,12 +911,31 @@ class DE1 (Singleton):
 
     async def upload_json_v2_profile(self, profile: Union[bytes,
                                                           bytearray,
-                                                          str]):
+                                                          str]) -> dict:
+        pbf = await self._process_json_v2_profile_inner(profile,
+                                                        upload_to_de1=True)
+        return {'id': pbf.id, 'fingerprint': pbf.fingerprint}
+
+    async def store_json_v2_profile(self, profile: Union[bytes,
+                                                         bytearray,
+                                                         str]) -> dict:
+        pbf = await self._process_json_v2_profile_inner(profile,
+                                                        upload_to_de1=False)
+        return {'id': pbf.id, 'fingerprint': pbf.fingerprint}
+
+    async def _process_json_v2_profile_inner(
+            self, profile: Union[bytes,
+                           bytearray,
+                           str], upload_to_de1=True) -> ProfileByFrames:
+
         pbf = ProfileByFrames().from_json(profile)
-        await self.upload_profile(pbf)
+        if upload_to_de1:
+            await self.upload_profile(pbf)
+        else:
+            self._fingerprint_profile_by_frames(pbf)
         async with aiosqlite.connect(config.database.FILENAME) as db:
             await db_insert.profile(pbf, db, time.time())
-            logger.info("Returned from db insert")
+        return pbf
 
     # "Internal" version
 
@@ -967,6 +996,9 @@ class DE1 (Singleton):
                         done = await self.start_notifying(cuuid)
                         # await done.wait()
 
+                # NB: Fingerprint code is replicated in
+                #     _fingerprint_profile_by_frames()
+
                 self._latest_profile = None
                 bytes_for_fingerprint = bytearray()
 
@@ -985,12 +1017,18 @@ class DE1 (Singleton):
                 profile._fingerprint = hashlib.sha1(
                     bytes_for_fingerprint).hexdigest()
 
+                async with aiosqlite.connect(config.database.FILENAME) as db:
+                    await db_insert.persist_last_profile(profile, db)
+                logger.info(f"Selected profile ID: {profile.id}")
+
                 if profile.number_of_preinfuse_frames is not None:
                     self._number_of_preinfuse_frames = \
                         profile.number_of_preinfuse_frames
 
                 if profile.tank_temperature is not None \
                     and override_tank_temperature:
+                    logger.info(
+                        f"Setting tank temp to {profile.tank_temperature}")
                     await self.write_and_read_back_mmr0x80(
                         addr_low=MMR0x80LowAddr.TANK_TEMP,
                         value=profile.tank_temperature
@@ -1021,6 +1059,30 @@ class DE1 (Singleton):
         finally:
             profile_upload_stopped.set()
 
+    def _fingerprint_profile_by_frames(self, profile: ProfileByFrames):
+
+        # NB: Fingerprint code is replicated from
+        #     _upload_profile()
+
+        bytes_for_fingerprint = bytearray()
+
+        # await self.write_packed_attr(profile.header_write())
+        bytes_for_fingerprint += profile.header_write().as_wire_bytes()
+        for frame in profile.shot_frame_writes():
+            # await self.write_packed_attr(frame)
+            bytes_for_fingerprint += frame.as_wire_bytes()
+        for frame in profile.ext_shot_frame_writes():
+            # await self.write_packed_attr(frame)
+            bytes_for_fingerprint += frame.as_wire_bytes()
+        # await self.write_packed_attr(profile.shot_tail_write())
+        bytes_for_fingerprint \
+            += profile.shot_tail_write().as_wire_bytes()
+
+        profile._fingerprint = hashlib.sha1(
+            bytes_for_fingerprint).hexdigest()
+
+
+
     async def write_and_read_back_mmr0x80(self, addr_low: MMR0x80LowAddr,
                                           value: float):
         if not addr_low.can_write:
@@ -1808,6 +1870,8 @@ class FeatureFlag:
     # 1260  2eae3cc 2021-05-06  Skip-to-next
     # 1265  224a312 2021-06-30
     # 1283  d8e169b 2021-09-30  Rinse (flush) control, Hot water flow (?)
+    #                               (Has start-up temperature issues)
+    # 1293  d014017 2021-12-22  "fixes problem with scheduler"
 
     @property
     def fw_version(self):
@@ -1893,3 +1957,10 @@ class FeatureFlag:
         else:
             return False    # Seemingly not supported in 1283
 
+    @property
+    def sched_idle(self):
+        if self.fw_version is None:
+            return None
+        else:
+            return self.fw_version >= 1293
+
