@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021, 2022 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -50,7 +50,7 @@ def validate_patch_return_targets(resource: Resource,
         patch = { None: patch }
     else:
         raise DE1APITypeError(
-            "Mapping and patch inconsistent, "
+            "Validate: Mapping and patch inconsistent, "
             "dict with dict, IsAt with raw value "
             f"not {type(mapping)} with {type(patch)}"
         )
@@ -116,9 +116,13 @@ def _validate_patch_inner(patch: dict, mapping: dict, path: str, targets: dict):
                 type_tuple = (*type_tuple, int,)
 
             if not isinstance(new_value, type_tuple):
+                try:
+                    type_str = mapping_value.v_type.__name__
+                except AttributeError:
+                    type_str = repr(mapping_value.v_type)
                 raise DE1APITypeError(
-                    f"Expected {mapping_value.v_type.__name__} value "
-                    f"at {this_path}:, not {new_value}"
+                    f"Expected {type_str} value at {this_path}, "
+                    f"not {new_value} {type(new_value)}"
                 )
 
             if mapping_value.requires_connected_de1:
