@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021, 2022 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -16,7 +16,7 @@ import pyDE1
 from pyDE1.scale import Scale
 from pyDE1.scale.events import ScaleWeightUpdate, ScaleButtonPress
 
-logger = pyDE1.getLogger('Scale.AtomaxIIScale')
+logger = pyDE1.getLogger('Scale.AtomaxSkaleII')
 
 
 class AtomaxSkaleII(Scale):
