@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021-2022 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -7,35 +7,29 @@ SPDX-License-Identifier: GPL-3.0-only
 """
 
 import asyncio
-import enum
-import logging
-
-
-from typing import Optional, List, Tuple, Coroutine, Callable
+import time
+from typing import Optional, List, Callable
 from uuid import UUID
 
+import pyDE1
+from pyDE1.config import config
+from pyDE1.de1 import DE1
 from pyDE1.de1.c_api import API_MachineStates
 from pyDE1.de1.events import StateUpdate
-from pyDE1.scanner import BleakScannerWrapped
-
-from pyDE1.config.bluetooth import CONNECT_TIMEOUT
 from pyDE1.dispatcher.resource import ConnectivityEnum
-from pyDE1.exceptions import DE1NoAddressError, DE1APIValueError
-from pyDE1.scale import Scale, ScaleError, scale_factory, \
-    recognized_scale_prefixes
 from pyDE1.event_manager import SubscribedEvent
-from pyDE1.scale.events import ScaleWeightUpdate, ScaleTareSeen, \
-    WeightAndFlowUpdate
-from pyDE1.scanner import DiscoveredDevices, find_first_matching
+from pyDE1.event_manager.events import ConnectivityState
+from pyDE1.exceptions import DE1NoAddressError, DE1APIValueError
+from pyDE1.scale import Scale, scale_factory, recognized_scale_prefixes
+from pyDE1.scale.events import (
+    ScaleWeightUpdate, ScaleTareSeen, WeightAndFlowUpdate
+)
+from pyDE1.scanner import (
+    BleakScannerWrapped, DiscoveredDevices, find_first_matching
+)
 from pyDE1.singleton import Singleton
 
-from pyDE1.de1 import DE1
-
-logger = logging.getLogger('ScaleProcessor')
-
-
-class ScaleProcessorError (ScaleError):
-    pass
+logger = pyDE1.getLogger('Scale.Processor')
 
 
 class ScaleProcessor (Singleton):
@@ -54,7 +48,7 @@ class ScaleProcessor (Singleton):
     # def __init__(self):
     #     pass
 
-    def _singleton_init(self, scale: Optional[Scale]=None):
+    def _singleton_init(self):
 
         self._scale: Optional[Scale] = None
         self._scale_weight_update_id: Optional[UUID] = None
@@ -65,7 +59,7 @@ class ScaleProcessor (Singleton):
         self._history_weight: List[float] = []
         self._history_lock = asyncio.Lock()
         # set_scale needs _history_lock
-        asyncio.create_task(self.set_scale(scale))
+        self._scale = None
         self._event_weight_and_flow_update = SubscribedEvent(self)
 
         # init of Estimator checks that the targets are present
@@ -80,11 +74,20 @@ class ScaleProcessor (Singleton):
         self._median_flow_time: float = 0
         self._estimators = [
             CurrentWeight(self, '_current_weight'),
-            AverageFlow(self, '_average_flow', 10),
-            MedianWeight(self, '_median_weight', 10),
-            MedianFlow(self, '_median_flow', 10, 5),
+            AverageFlow(self, '_average_flow', 11),
+            MedianWeight(self, '_median_weight', 11),
+            MedianFlow(self, '_median_flow', 11, 5),
         ]
 
+        # This is a bit tricky, as self._scale is None
+        bare_scale = Scale()
+        asyncio.create_task(
+            bare_scale._event_connectivity.publish(
+                bare_scale._connectivity_change(
+                    arrival_time=time.time(),
+                    state=ConnectivityState.DISCONNECTED))
+        )
+
     @property
     def scale(self):
         return self._scale
@@ -166,7 +169,11 @@ class ScaleProcessor (Singleton):
     # Self-contained call for API
     #
 
-    async def first_if_found(self, doit: bool):
+    async def first_if_found(self, doit: bool, warn_deprecated=True):
+        if warn_deprecated:
+            logger.warning(
+                "Use of 'first_if_found' is deprecated in favor of "
+                "setting {'id': 'scan'}")
         if self.scale and self.scale.is_connected:
             logger.warning(
                 "first_if_found requested, but already connected. "
@@ -188,6 +195,10 @@ class ScaleProcessor (Singleton):
         """
         logger.info(f"Request to replace scale with {ble_device_id}")
 
+        if ble_device_id == 'scan':
+            await self.first_if_found(doit=True, warn_deprecated=False)
+            return
+
         # TODO: Need to make distasteful assumption that the id is the address
         try:
             if self.scale_address == ble_device_id:
@@ -209,7 +220,8 @@ class ScaleProcessor (Singleton):
                     logger.warning(f"No record of {ble_device_id}, initiating scan")
                     # TODO: find_device_by_filter doesn't add to DiscoveredDevices
                     ble_device = await BleakScannerWrapped.find_device_by_address(
-                        ble_device_id, timeout=CONNECT_TIMEOUT)
+                        ble_device_id,
+                        timeout=config.bluetooth.CONNECT_TIMEOUT)
                 if ble_device is None:
                     raise DE1NoAddressError(
                         f"Unable to find device with id: '{ble_device_id}'")
@@ -255,10 +267,13 @@ class ScaleProcessor (Singleton):
 
     async def _reset(self):
         async with self._history_lock:
-            self._history_time = []
-            self._history_weight = []
-            # probably should clear any pending updates
-            # as they may be pre-tare
+            self._reset_have_lock()
+
+    def _reset_have_lock(self):
+        self._history_time = []
+        self._history_weight = []
+        # TODO: Perhaps should clear any pending updates
+        #       as they may be pre-tare
 
     @property
     def _history_available(self):
@@ -286,6 +301,20 @@ class ScaleProcessor (Singleton):
             # "Acquiring a lock is fair: the coroutine that proceeds
             #  will be the first coroutine that started waiting on the lock."
             async with self._history_lock:
+                # Detect a gap in reporting being "too long"
+                # (typically from a disconnect/reconnect)
+                # A skip of three at 150 ms per update with the Skale II
+                TOO_LONG = 0.8 # seconds
+                try:
+                    if ((dt := swu.scale_time
+                               - scale_processor._history_time[-1]) > TOO_LONG):
+                        logger.warning(
+                            "Resetting scale due to gap in reports: "
+                            f"{dt:0.3f} > {TOO_LONG} s")
+                        scale_processor._reset_have_lock()
+                        return
+                except IndexError:
+                    pass  # (No elements in the history list)
                 scale_processor._history_time.append(swu.scale_time)
                 scale_processor._history_weight.append(swu.weight)
                 # Unlikely, but possibly the case that history_max shrunk
