@@ -1,5 +1,5 @@
 """
-Copyright © 2021 Jeff Kletsky. All Rights Reserved.
+Copyright © 2021-2022 Jeff Kletsky. All Rights Reserved.
 
 License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
@@ -7,33 +7,25 @@ SPDX-License-Identifier: GPL-3.0-only
 """
 
 import asyncio
-import logging
 import time
-import gc
-
 from typing import Optional, Callable, Coroutine, Union
 
+import aiosqlite
 from bleak.backends.device import BLEDevice
-from bleak.backends.scanner import AdvertisementData
 from bleak.backends.service import BleakGATTServiceCollection
 
+import pyDE1
+import pyDE1.shutdown_manager as sm
 from pyDE1.bleak_client_wrapper import BleakClientWrapped
-from pyDE1.de1.c_api import API_MachineStates
-from pyDE1.scanner import BleakScannerWrapped
-from pyDE1.exceptions import DE1APIValueError, DE1NoAddressError
+from pyDE1.config import config
 from pyDE1.dispatcher.resource import ConnectivityEnum
 from pyDE1.event_manager import SubscribedEvent
 from pyDE1.event_manager.events import ConnectivityState, ConnectivityChange
+from pyDE1.exceptions import DE1APIValueError, DE1NoAddressError
 from pyDE1.scale.events import ScaleWeightUpdate, ScaleTareSeen
-from pyDE1.scanner import _registered_ble_prefixes, DiscoveredDevices
-from pyDE1.config.bluetooth import CONNECT_TIMEOUT
+from pyDE1.scanner import _registered_ble_prefixes
 
-from pyDE1.de1 import DE1
-
-logger = logging.getLogger('Scale')
-
-# TODO: Can/should this be related to the class?
-#       If so, how should subclasses respond?
+logger = pyDE1.getLogger('Scale')
 
 # Used for factory and for BLE detection and filtering
 _prefix_to_constructor = dict()
@@ -43,21 +35,8 @@ _recognized_scale_prefixes = set()
 def recognized_scale_prefixes():
     return _recognized_scale_prefixes.copy()
 
-
-class ScaleError(RuntimeError):
-    def __init__(self, *args, **kwargs):
-        super(ScaleError, self).__init__(args, kwargs)
-
-
-class ScaleNoAddressError(ScaleError):
-    def __init__(self, *args, **kwargs):
-        super(ScaleNoAddressError, self).__init__(args, kwargs)
-
-
-class ScaleNotConnectedError(ScaleError):
-    def __init__(self, *args, **kwargs):
-        super(ScaleNotConnectedError, self).__init__(args, kwargs)
-
+# TODO: Experimentaly confirm that weight and mass-flow estimates
+#       are reasonably time aligned - NB: DE1.fall_time
 
 class Scale:
 
@@ -88,8 +67,8 @@ class Scale:
 
         self._ready = asyncio.Event()
 
-        self._reconnect_delay = 0
-        self._logging_reconnect = True
+        self._reconnect_count = 0
+        self._log_reconnect_attempts = True
 
         asyncio.get_event_loop().create_task(
             self._event_connectivity.publish(
@@ -109,18 +88,13 @@ class Scale:
             '_event_tare_seen',
         )
 
-        # TODO: Think about how to manage "tare seen"
+        # TODO: Think about how to manage a "tare seen" event
+        #       and what the use cases for it would be.
         #       Could use asyncio.Event(), but what are the states
         #       and how do you "release" if it never arrives?
-        #       It seems like the use case would be:
-        #           request tare
-        #           wait for tare
-        #           if seen:
-        #               continue
-        #           else:
-        #               do something else
+
         self._tare_requested = False
-        self._period_estimator = self.PeriodEstimator(self._nominal_period)
+        self._period_estimator = PeriodEstimator(self)
 
         # Don't need to await this on instantiation
         asyncio.get_event_loop().create_task(
@@ -159,10 +133,11 @@ class Scale:
     async def connect(self, timeout: Optional[float] = None):
 
         if timeout is None:
-            timeout = CONNECT_TIMEOUT
+            timeout = config.bluetooth.CONNECT_TIMEOUT
 
         class_name = type(self).__name__
-        logger.info(f"Connecting to {class_name} at {self.address}")
+        if self._log_reconnect_attempts:
+            logger.info(f"Connecting to {class_name} at {self.address}")
 
         assert self._bleak_client is not None
 
@@ -193,11 +168,10 @@ class Scale:
                 # At least BlueZ doesn't like concurrent connection requests
                 asyncio.create_task(self.standard_initialization())
 
-                # TODO: Does the ScaleProcessor get properly reset?
-
             else:
-                logger.error(
-                    f"Connection failed to {class_name} at {self.address}")
+                if self._log_reconnect_attempts:
+                    logger.error(
+                        f"Connection failed to {class_name} at {self.address}")
                 await self._notify_not_ready()
                 await self._event_connectivity.publish(
                     self._connectivity_change(
@@ -213,6 +187,7 @@ class Scale:
         await self.start_sending_weight_updates()
         if self.supports_button_press:
             await self.start_sending_button_updates()
+        await self._restore_period_from_db()
         if not hold_notification:
             await self._notify_ready()
 
@@ -278,8 +253,6 @@ class Scale:
                     arrival_time=time.time(),
                     state=ConnectivityState.DISCONNECTED))
 
-    # TODO: Decide how to handle  self._disconnected_callback
-
     @property
     def is_connected(self):
         if self._bleak_client is None:
@@ -435,15 +408,11 @@ class Scale:
         #
         # logger.info(f"0x{id(self):x} is_finalized: {gc.is_finalized(self)}")
 
-    # TODO: Decide how to handle  self._disconnected_callback
-    #   disconnected_callback (callable): Callback that will be scheduled in the
-    #   event loop when the client is disconnected. The callable must take one
-    #   argument, which will be this client object.
+    #  disconnected_callback (callable): Callback that will be scheduled in the
+    #  event loop when the client is disconnected. The callable must take one
+    #  argument, which will be this client object.
 
     # The callback seems to be expected to be a "plain" function
-    #     task.add_done_callback(
-    #         lambda _: self._disconnected_callback(self)
-    #     )
 
     def _create_disconnect_callback(self) -> Callable:
         scale = self
@@ -464,44 +433,50 @@ class Scale:
                         state=ConnectivityState.DISCONNECTED)),
                 return_exceptions=True)
             )
-            # TODO: Don't try to reconnect on shutdown
+
             if not client.willful_disconnect:
                 asyncio.get_event_loop().create_task(self._reconnect())
 
         return disconnect_callback
 
     def _reset_reconnect(self):
-        self._reconnect_delay = 0
-        self._logging_reconnect = True
+        self._reconnect_count = 0
+        self._log_reconnect_attempts = True
 
     async def _reconnect(self):
         """
-        Will try immediately, then 1, 2, 3, ..., 10, 10, ... seconds later
-        Each try includes default scan time (10 sec), then a delay
-        TODO: Is there a better pattern?
+        Will try immediately config.bluetooth.RECONNECT_RETRY_COUNT times
+            of CONNECT_TIMEOUT each. It will then stop logging and retry
+            with RECONNECT_GAP seconds in between attempts
         """
-        if DE1().current_state == API_MachineStates.Sleep:
-            logger.info("DE1 is sleeping, not retrying to connect")
+
+        if sm.shutdown_underway.is_set():
             return
+
+        if self._reconnect_count >= config.bluetooth.RECONNECT_RETRY_COUNT:
+            await asyncio.sleep(config.bluetooth.RECONNECT_GAP)
+
+        if self._log_reconnect_attempts:
+            class_name = type(self).__name__
+            logger.info(
+                f"Will try reconnecting to {class_name} at {self.address}")
+
         # Workaround for https://github.com/hbldh/bleak/issues/376
         self._bleak_client.services = BleakGATTServiceCollection()
-        class_name = type(self).__name__
-        if self._logging_reconnect:
-            logger.info(
-                f"Will try reconnecting to {class_name} at {self.address} "
-                f"after waiting {self._reconnect_delay} seconds.")
-        await asyncio.sleep(self._reconnect_delay)
 
         await self.connect()
         if self.is_connected:
             self._reset_reconnect()
+
         else:
-            if self._reconnect_delay <= 10:
-                self._reconnect_delay = self._reconnect_delay +1
-            if self._reconnect_delay == 10:
+            if self._reconnect_count <= config.bluetooth.RECONNECT_RETRY_COUNT:
+                self._reconnect_count = self._reconnect_count + 1
+            if self._reconnect_count == config.bluetooth.RECONNECT_RETRY_COUNT:
                 logger.info("Suppressing further reconnect messages. "
-                            "Will keep trying at 10-second intervals.")
-                self._logging_reconnect = False
+                            "Will keep trying at {}-second intervals.".format(
+                    config.bluetooth.RECONNECT_GAP)
+                )
+                self._log_reconnect_attempts = False
             asyncio.get_event_loop().create_task(
                 self._reconnect(), name='ReconnectScale')
 
@@ -514,57 +489,45 @@ class Scale:
         self._nominal_period = value
         self._period_estimator.reset(self._nominal_period)
 
-    # Inner class
-    class PeriodEstimator:
-        """
-        Estimate inter-arrival period from stream of arrivals
+    async def _persist_period_to_db(self):
+        if not self.address:
+            raise DE1NoAddressError(
+                "Can't persist scale period without a scale address")
+        async with aiosqlite.connect(config.database.FILENAME) as db:
+            sql = "INSERT OR REPLACE INTO persist_hkv " \
+                  "(header, key, value) " \
+                  "VALUES " \
+                  "(:header, :key, :value) "
+            await db.execute(sql, {
+                'header': 'scale.period',
+                'key': self.address,
+                'value': self.nominal_period,
+            })
+            await db.commit()
 
-        Presently just an exponential moving average
-
-        Skale II usually "bulks up" two or more reports on a 150-ms clock
-        300 ms burbles aren't uncommon. A "normal" 50-ms stretch before its
-        other half arrives would generate a (50/100) * k change.
-        The other half then would generate (-100/100) * k change
-        So k on the order of 1/1000 should be reasonable (10 sec, ~1 min settle)
-        k of 1/10000 would be even better (100 sec, 10 min settle)
-        Another way to look at this is 600 ms error / 600 s measurement ~ 0.1%
-
-        Hand-in-hand with this is how long to consider a gap vs. a burble
-        Nearly 5% of 150-ms windows from a SkaleII had 3 reports.
-        Up to 6 in a window were observed. It dropped to 0.1% at 4 reports
-        per window. Ignoring too many of these can lead to the estimate being off.
-        Based on this, 300 ms (two periods) seems too short.
-        300 + 150/2 = 375 ms is probably OK.
-        450 + 150/2 = 525 ms is probablu conservative
-        Try 500 ms to be reasonable.
-        """
-
-        def __init__(self, nominal_period: float):
-
-            # TODO: How to update this for subclass changes?
-
-            self._k = 1/10000
-            self._ma = nominal_period
-            self._too_long = 0.5  # seconds before considered a gap
-
-            self._log_every_n = 1000
-            self._n_counter = 0
-
-        def reset(self, nominal_period: float):
-            self._ma = nominal_period
-
-        async def process_arrival(self, delta_arrival_time: float):
-
-            if delta_arrival_time < self._too_long:
-                self._ma = ((1 - self._k) * self._ma) \
-                           + (self._k * delta_arrival_time)
-                self._n_counter += 1
-                if self._n_counter >= self._log_every_n:
-                    self._n_counter = 0
-                    logger.debug(f"Scale period: {self._ma}")
-
-
-    # TODO: Deal with connectivity as a mixin
+    async def _restore_period_from_db(self):
+        if not self.address:
+            raise DE1NoAddressError(
+                "Can't restore scale period without a scale address")
+        async with aiosqlite.connect(config.database.FILENAME) as db:
+            sql = "SELECT value FROM persist_hkv " \
+                  "WHERE header = :header AND key = :key"
+            cur = await db.execute(sql, {
+                'header': 'scale.period',
+                'key': self.address,
+            })
+            row = await cur.fetchone()
+            if row and row[0]:
+                val = float(row[0])
+                logger.info(
+                    "Loading scale-period estimate of "
+                    f"{val:.5f} from database")
+                self._estimated_period = val
+                self._period_estimator.reset(val)
+            else:
+                logger.info(
+                    "No previous scale-period estimate for "
+                    f"{self.address} found")
 
     # For API
     @property
@@ -584,12 +547,67 @@ class Scale:
         _registered_ble_prefixes.add(prefix)
 
 
+# Used by Scale instances
+class PeriodEstimator:
+    """
+    Estimate inter-arrival period from stream of arrivals
+
+    Presently just an exponential moving average
+
+    Skale II usually "bulks up" two or more reports on a 150-ms clock
+    300 ms burbles aren't uncommon. A "normal" 50-ms stretch before its
+    other half arrives would generate a (50/100) * k change.
+    The other half then would generate (-100/100) * k change
+    So k on the order of 1/1000 should be reasonable (10 sec, ~1 min settle)
+    k of 1/10000 would be even better (100 sec, 10 min settle)
+    Another way to look at this is 600 ms error / 600 s measurement ~ 0.1%
+
+    Hand-in-hand with this is how long to consider a gap vs. a burble
+    Nearly 5% of 150-ms windows from a SkaleII had 3 reports.
+    Up to 6 in a window were observed. It dropped to 0.1% at 4 reports
+    per window. Ignoring too many of these can lead to the estimate being off.
+    Based on this, 300 ms (two periods) seems too short.
+    300 + 150/2 = 375 ms is probably OK.
+    450 + 150/2 = 525 ms is probablu conservative
+    Try 500 ms to be reasonable.
+    """
+
+    def __init__(self, my_scale):
+
+        # TODO: How to update this PeriodEstimator for subclass changes?
+
+        self._scale = my_scale
+
+        self._k = 1/10000   # tau ~ 17 min at 10 samples/sec
+        self._ma = self._scale.nominal_period
+        self._too_long = 0.5  # seconds before considered a gap
+
+        self._persist_every_n = 1000  # about 100 seconds
+        self._n_counter = 0
+
+    def reset(self, nominal_period: float):
+        self._ma = nominal_period
+
+    async def process_arrival(self, delta_arrival_time: float):
+
+        if delta_arrival_time < self._too_long:
+            self._ma = ((1 - self._k) * self._ma) \
+                       + (self._k * delta_arrival_time)
+            self._scale.nominal_period = self._ma
+            self._n_counter += 1
+            if self._n_counter >= self._persist_every_n:
+                self._n_counter = 0
+                logger.debug(f"Scale period: {self._ma}")
+                await self._scale._persist_period_to_db()
+
+
+
 def scale_factory(ble_device: BLEDevice)-> Scale:
     constructor = None
     try:
         constructor = _prefix_to_constructor[ble_device.name]
     except KeyError:
-        for prefix in _prefix_to_constructor.values():
+        for prefix in _prefix_to_constructor.keys():
             if ble_device.name.startswith(prefix):
                 constructor = _prefix_to_constructor[prefix]
     if constructor is None:
